{% load static reactor %}
<link rel="stylesheet" href="{% static "reactor/reactor.css" %}">
<script defer src="{% static "reactor/morphdom.min.js" %}"></script>
{% if include_tlinks %}
  <script defer src="{% static "reactor/turbolinks.min.js" %}"></script>
{% endif %}
<script defer src="{% static "reactor/reactor.js" %}"></script>
